@@ -11,11 +11,13 @@
 #define KM_LARGE	0x0010u
 #define KM_NOLOCKDEP	0x0020u
 
-typedef struct kmem_zone {
-	int	zone_unitsize;	/* Size in bytes of zone unit           */
-	char	*zone_name;	/* tag name                             */
-	int	allocated;	/* debug: How many currently allocated  */
-} kmem_zone_t;
+struct kmem_cache {
+	int		cache_unitsize;	/* Size in bytes of cache unit */
+	int		allocated;	/* debug: How many allocated? */
+	unsigned int	align;
+	const char	*cache_name;	/* tag name */
+	void		(*ctor)(void *);
+};
 
 typedef unsigned int __bitwise gfp_t;
 
@@ -26,15 +28,24 @@ typedef unsigned int __bitwise gfp_t;
 
 #define __GFP_ZERO	(__force gfp_t)1
 
-extern kmem_zone_t *kmem_zone_init(int, char *);
-extern void	*kmem_cache_alloc(kmem_zone_t *, gfp_t);
-extern void	*kmem_cache_zalloc(kmem_zone_t *, gfp_t);
-extern int	kmem_zone_destroy(kmem_zone_t *);
+struct kmem_cache * kmem_cache_create(const char *name, unsigned int size,
+		unsigned int align, unsigned int slab_flags,
+		void (*ctor)(void *));
+
+static inline struct kmem_cache *
+kmem_cache_init(unsigned int size, const char *name)
+{
+	return kmem_cache_create(name, size, 0, 0, NULL);
+}
+
+extern void	*kmem_cache_alloc(struct kmem_cache *, gfp_t);
+extern void	*kmem_cache_zalloc(struct kmem_cache *, gfp_t);
+extern int	kmem_cache_destroy(struct kmem_cache *);
 
 static inline void
-kmem_cache_free(kmem_zone_t *zone, void *ptr)
+kmem_cache_free(struct kmem_cache *cache, void *ptr)
 {
-	zone->allocated--;
+	cache->allocated--;
 	free(ptr);
 }
 
